package net.weasel.Vegetation;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.block.Biome;
import org.bukkit.block.Block;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.scheduler.BukkitScheduler;

public class Vegetation extends JavaPlugin
{
	public boolean debugging = false;
	public static Random generator = new Random();
	public static BukkitScheduler timer;
	public static int tTask;
	public static Integer timerDelay;
	public static String pluginName = "Vegetation";
	public static String pluginVersion = "1.0";
	public static boolean enableGrass = true;
	public static float grassPerCycle = 0;
	public static float plantsPerCycle = 0;
	public static float grassTick = 0;
	public static float plantTick = 0;
	public static int growthRange = 0;
	public static int verticalRadius = 5;

	public static boolean enablePlants = true;
	public static boolean enablePumpkins = true;
	public static boolean enableFlowers = true;
	public static boolean enableFungi = true;
	public static boolean enableCacti = true;
	public static boolean enableCanes = true;
		
	public static boolean growForestBiome = true;
	public static boolean growRainforestBiome = true;
	public static boolean growShrublandBiome = true;
	public static boolean growSavannahBiome = true;
	public static boolean growPlainsBiome = true;
	public static boolean growSeasonalForestBiome = true;
	
	public static String pluginIni = "plugins/Vegetation/Settings.ini";
	
	public static List<Block> grassList = new ArrayList<Block>();
	public static List<Block> plantList = new ArrayList<Block>();
	
	@Override
    public void onEnable() 
    {
		if( new File("plugins/Vegetation/").exists() == false )
		{
			logOutput( "Settings folder does not exist - creating it.. ");
			
			boolean chk = new File("plugins/Vegetation/").mkdir();
			
			if( chk )
				logOutput( "Successfully created folder." );
			else
				logOutput( "Unable to create folder!" );
		}	
		
		if( new File(pluginIni).exists() == false )
		{
			logOutput( "Settings file does not exist - creating a new one.. ");
			
			if( createIniFile() == true )
				logOutput( "Created successfully." );
			else
				logOutput( "Unable to create ini file!" );
		}

		if( new File("plugins/Vegetation/vdebug.txt").exists() == true ) debugging = true;
		
        logOutput( pluginName + " v" + pluginVersion + " enabled." );
        
        timer = this.getServer().getScheduler();

        enableGrass = getBooleanSetting( "enableGrass", true );
        enablePlants = getBooleanSetting( "enablePlants", true );
    	enablePumpkins = getBooleanSetting( "enablePumpkins", true );
    	enableFlowers = getBooleanSetting( "enableFlowers", true );
    	enableFungi = getBooleanSetting( "enableFungi", true );
    	enableCacti = getBooleanSetting( "enableCacti", true );
    	enableCanes = getBooleanSetting( "enableCanes", true );

    	growForestBiome = getBooleanSetting( "growForestBiome", true );
    	growRainforestBiome = getBooleanSetting( "growRainforestBiome", true );
    	growShrublandBiome = getBooleanSetting( "growShrublandBiome", true );
    	growSavannahBiome = getBooleanSetting( "growSavannahBiome", true );;
    	growPlainsBiome = getBooleanSetting( "growPlainsBiome", true );
    	growSeasonalForestBiome = getBooleanSetting( "growSeasonalForestBiome", true );

    	verticalRadius = getIntSetting( "verticalRadius", 5 );
        timerDelay = getIntSetting( "growDelay", 10 )*1000;
        growthRange = getIntSetting( "growthRange", 100 );
        grassPerCycle = getFloatSetting( "grassPerCycle", Float.valueOf("20") );
        plantsPerCycle = getFloatSetting( "plantsPerCycle", Float.valueOf("0.5") );
        
        if( enableGrass || enablePlants )
        {
	        tTask = timer.scheduleAsyncRepeatingTask(this, new Runnable()
	        {
				@Override
				public void run() 
				{
					World currentWorld = null;
				 
					for( int W = 0; W < getServer().getWorlds().size(); W++ )
					{
						currentWorld = getServer().getWorlds().get(W);
					
						if( getServer().getOnlinePlayers().length > 0 && currentWorld.getTime() < 12000 )
						{
							grassTick += grassPerCycle;
								
							if( grassTick >= 1 )
							{
								if( enableGrass == true ) growTheGrass( (int)grassPerCycle );
								grassTick = 0;
							}
		
							plantTick += plantsPerCycle;
							
							if( plantTick >= 1 )
							{
								if( enablePlants == true ) growThePlants( (int) plantsPerCycle );
								plantTick = 0;
							}
						}
					}
				
	        	}
	        }, (timerDelay/1000)*20, (timerDelay/1000)*20 );

	        logOutput( "Growth cycle set to every "+(timerDelay/1000)+" seconds.");
	        logOutput( "Growth limit: "+growthRange+"(XZ) and " + verticalRadius + "(Y) blocks of players.");
	        logOutput( "Grass is " + ( enableGrass ? "ENABLED" : "DISABLED" ) + " and plants are " + ( enablePlants ? "ENABLED" : "DISABLED" ) );
	        logOutput( grassPerCycle + " grass & " + plantsPerCycle + " plants, each cycle." );
	    }
        else
        {
        	logOutput( "No vegetation is enabled. Disabling plugin." );
            this.getPluginLoader().disablePlugin(this);
        }
    }
    
    @Override
    public void onDisable() 
    {
    	timer.cancelTask(tTask);
        logOutput( "Plugin disabled: "+pluginName+" version "+pluginVersion);
    }
    
    public String[] getSetting( String which )
    {
        return( getSettingValue(pluginIni, which, "10", "" ) );
    }

    public boolean getBooleanSetting( String which, boolean dValue )
    {
    	boolean retVal;
    	
    	String dString = ( dValue ? "true" : "false" );
    	
        if( getSettingValue(pluginIni, which, dString, "" )[0].equalsIgnoreCase("true") )
        	retVal = true;
        else
        	retVal = false;
        
        return( retVal );
    }
    
    public Integer getIntSetting( String item, Integer dValue )
    {
    	Integer retVal = Integer.parseInt( getSettingValue(pluginIni, item, dValue.toString(), "" )[0]);
    	return retVal;
    }

    public Float getFloatSetting( String item, Float dValue )
    {
    	Float retVal = Float.valueOf(getSettingValue(pluginIni, item, dValue.toString(), "" )[0]);
    	return retVal;
    }

    public boolean createIniFile()
    {
    	boolean retVal = false;
    	
    	try 
    	{
			FileWriter outFile = new FileWriter(pluginIni);
			PrintWriter outP = new PrintWriter(outFile);
			
			outP.println( "growDelay=10" );
			outP.println( "growthRange=100" );
			outP.println( "verticalRadius=5" );
			outP.println( "enableGrass=true" );
			outP.println( "enablePlants=true" );
			outP.println( "enableFlowers=true" );
			outP.println( "enableFungi=true" );
			outP.println( "enablePumpkins=true" );
			outP.println( "enableCacti=true" );
			outP.println( "enableCanes=true" );
			outP.println( "grassPerCycle=20" );
			outP.println( "plantsPerCycle=0.5" );
			outP.println( "growForestBiome=true" );
			outP.println( "growRainforestBiome=true" );
			outP.println( "growShrublandBiome=true" );
			outP.println( "growSavannahBiome=true" );
			outP.println( "growPlainsBiome=true" );
			outP.println( "growSeasonalForestBiome=true" );
			
			outP.close();
			retVal = true;
		} 
    	catch (IOException e) 
    	{
			logOutput( "Error writing to ini file." );
			retVal = false;
			e.printStackTrace();
		}
		
		return retVal;
    }
    
	public boolean onCommand(CommandSender sender, Command command, String commandLabel, String[] args) 
    {
    	String pCommand = command.getName().toLowerCase();
    	
    	if( pCommand.equals("mow") )
    	{
    		mowTheGrass( (Player) sender );
    		sender.sendMessage( "Rrrrrrrrr...." );
    	}
    	
    	if( pCommand.equals("growplants") )
    	{
    		growThePlants( (int) plantsPerCycle );
    		sender.sendMessage( "Growing the plants.." );
    	}
    	
    	if( pCommand.equals("growgrass") )
    	{
    		growTheGrass( (int) grassPerCycle );
    		sender.sendMessage( "Growing the grass.." );
    	}
    	
    	if( pCommand.equals("overgrow") )
    	{
			sender.sendMessage( "Right away, Totoro-san! ^_^ Prepare for lag.." );
			
			for( int Q = 0; Q < 20; Q++ )
			{
				growTheGrass( (int) grassPerCycle );
				growThePlants( (int) plantsPerCycle );
			
				try 
				{
					Thread.sleep( 200 );
				}
				catch (InterruptedException e) 
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}

			sender.sendMessage( "Overgrow completed." );
    	}

    	if( pCommand.equals("grow") )
    	{
    		growThePlants( (int) plantsPerCycle );
    		growTheGrass( (int) grassPerCycle );
    		sender.sendMessage( "Growing everything.." );
    	}

    	return true;
    }
	
    public void growTheGrass( Integer howMany )
    {
    	Player player = null;
    	Integer I = 0;
    	int V = 0;
    	
		while( I < this.getServer().getOnlinePlayers().length ) 
		{
			player = this.getServer().getOnlinePlayers()[I];
			getGrassBlocks( player, growthRange );
			Block currentBlock = null;
			Integer R;
			
			for( Integer C = 0; C < howMany; C++ )
			{
				if( grassList.size() > 0 )
				{
					R = generator.nextInt(grassList.size());
					currentBlock = grassList.get(R);
	
					if( currentBlock != null )
					{
						V = currentBlock.getData() + 1;
						
						if( V > 10 ) V = 10;
		    
						if( debugging ) System.out.println( "Adjusting block " + currentBlock.getX() + "," + currentBlock.getY() + "," + currentBlock.getZ() );
						currentBlock.setData( (byte) V );
					}
				}
			}
			
			I++;
		}
    }

    public void growThePlants( Integer howMany )
    {
    	Player player = null;
    	Integer I = 0;
    	
		while( I < this.getServer().getOnlinePlayers().length ) 
		{
			Block currentBlock = null;
			player = this.getServer().getOnlinePlayers()[I];
			
			getPlantBlocks( player, growthRange );
		
			if( plantList.size() > 0 )
			{
				Integer R = generator.nextInt(plantList.size());
				currentBlock = plantList.get(R);
	
				if( currentBlock != null )
				{
					Block newBlock = getAdjacentEmptyDirtBlock(currentBlock);
				
					if( newBlock != null )
					{
						if( debugging ) System.out.println( "Spreading plant to block " + newBlock.getX() + "," + currentBlock.getY() + "," + currentBlock.getZ() );
						newBlock.setType( currentBlock.getType() );
						newBlock.setData((byte) 0);
					}
				}
			}
			
			I++;
		}
    }
    
    public void mowTheGrass( Player player )
    {
    	getGrassBlocks( player, growthRange );
    	
    	for( int X = 0; X < grassList.size(); X++ )
    	{
    		if( grassList.get(X).getData() > 1 ) grassList.get(X).setData( (byte) 1 );
    	}
    }
    
    public void getGrassBlocks( Player player, Integer radius )
    {
    	grassList.clear();
    	
    	Location loc = player.getLocation();
    	World world = player.getWorld();
    	
    	Integer pX = loc.getBlockX();
    	Integer pY = loc.getBlockY();
    	Integer pZ = loc.getBlockZ();
    	
    	Integer minX = pX - (radius / 2);
    	Integer minZ = pZ - (radius / 2);
    	Integer maxX = pX + (radius / 2);
    	Integer maxZ = pZ + (radius / 2);
    	
    	Integer maxY = pY + verticalRadius;
    	Integer minY = pY - verticalRadius;
    	
    	Block targetBlock = null;
    	
    	for( Integer Y = minY; Y < maxY ; Y++ )
    	{
	    	for( Integer X = minX; X < maxX; X++ )
	    	{
	    		for( Integer Z = minZ; Z < maxZ; Z++ )
	    		{
	    			targetBlock = world.getBlockAt(X, Y, Z);
	    			
	    			if( targetBlock.getData() >= 0 
	    			 && targetBlock.getData() <= 10 
	    			 && targetBlock.getTypeId() == 2
	    			 && (
	    					 ( targetBlock.getBiome() == Biome.FOREST && growForestBiome == true )
	    				  || ( targetBlock.getBiome() == Biome.RAINFOREST && growRainforestBiome == true )
	    				  || ( targetBlock.getBiome() == Biome.PLAINS && growPlainsBiome == true )
	    				  || ( targetBlock.getBiome() == Biome.SAVANNA && growSavannahBiome == true )
	    				  || ( targetBlock.getBiome() == Biome.SHRUBLAND&& growShrublandBiome == true )
	    				  || ( targetBlock.getBiome() == Biome.SEASONAL_FOREST && growSeasonalForestBiome == true )
	    			 	)
	    			)
	    			{
	    				grassList.add(targetBlock);
	    			}
	    		}
	    	}
    	}
    }

    public void getPlantBlocks( Player player, Integer radius )
    {
    	plantList.clear();

    	Location loc = player.getLocation();
    	World world = player.getWorld();

    	Integer pX = loc.getBlockX();
    	Integer pY = loc.getBlockY();
    	Integer pZ = loc.getBlockZ();

    	Integer minX = pX - (radius / 2);
    	Integer minY = pY - (radius / 2);
    	Integer maxX = pX + (radius / 2);
    	Integer maxY = pY + (radius / 2);

    	Block targetBlock = null;

    	for( Integer Z = (pZ-3); Z < (pZ+3); Z++ )
    	{
	    	for( Integer X = minX; X < maxX; X++ )
	    	{
	    		for( Integer Y = minY; Y < maxY; Y++ )
	    		{
	    			targetBlock = world.getBlockAt(X, Y, Z);

	    			if( 
	    				( targetBlock.getTypeId() == 37 && enableFlowers == true )
	    			 || ( targetBlock.getTypeId() == 38 && enableFlowers == true )
	    			 || ( targetBlock.getTypeId() == 39 && enableFungi == true )
	    			 || ( targetBlock.getTypeId() == 40 && enableFungi == true )
	    			 || ( targetBlock.getTypeId() == 86 && enablePumpkins == true ) 
	    			 || ( targetBlock.getTypeId() == 81 && enableCacti == true ) 
	    			 || ( targetBlock.getTypeId() == 83 && enableCanes == true ) 
	    			)
	    			{
	    				plantList.add(targetBlock);
	    			}
	    		}
	    	}
    	}
    }

    public Block getAdjacentEmptyDirtBlock(Block targetBlock )
    {
    	Integer X = targetBlock.getX();
    	Integer Y = targetBlock.getY();
    	Integer Z = targetBlock.getZ();
    	Integer T = targetBlock.getTypeId();
    	Block retVal = null, checkBlock = null;

    	checkBlock = targetBlock.getWorld().getBlockAt(X-1,Y-1,Z-1);

    	if( checkBlock.getTypeId() == T ) 
    		retVal = checkBlock;
    	else
    	{
    		checkBlock = targetBlock.getWorld().getBlockAt(X-1,Y-1,Z);
    		if( checkBlock.getTypeId() == T ) 
    			retVal = checkBlock;
    		else
    		{
    			checkBlock = targetBlock.getWorld().getBlockAt(X-1,Y-1,Z+1);
    			if( checkBlock.getTypeId() == T ) 
    				retVal = checkBlock;
    			else
    			{    	
    				checkBlock = targetBlock.getWorld().getBlockAt(X,Y-1,Z-1);
    				if( checkBlock.getTypeId() == T ) 
    					retVal = checkBlock;
    				else
    				{
    					checkBlock = targetBlock.getWorld().getBlockAt(X,Y-1,Z);
    					if( checkBlock.getTypeId() == T ) 
    						retVal = checkBlock;
    					else
    					{
    						checkBlock = targetBlock.getWorld().getBlockAt(X,Y-1,Z+1);
    						if( checkBlock.getTypeId() == T ) 
    							retVal = checkBlock;
    						else
    						{
    							checkBlock = targetBlock.getWorld().getBlockAt(X+1,Y-1,Z+1);
    							if( checkBlock.getTypeId() == T )
    								retVal = checkBlock;
    							else
    							{
    								checkBlock = targetBlock.getWorld().getBlockAt(X+1,Y-1,Z);
    								if( checkBlock.getTypeId() == T )
    									retVal = checkBlock;
    								else
    								{
    									checkBlock = targetBlock.getWorld().getBlockAt(X+1,Y-1,Z+1);
    									if( checkBlock.getTypeId() == T ) 
    										retVal = checkBlock;
    								}
    							}
    						}
    					}
    				}
    			}
    		}
    	}

    	if( retVal != null )
    	{
    		Integer rX = retVal.getX();
    		Integer rY = retVal.getY() + 1;
    		Integer rZ = retVal.getZ();
    		
    		retVal = targetBlock.getWorld().getBlockAt(rX,rY,rZ);
    	}
    	
    	return( retVal );
    }
    
    public String[] getSettingValue(String fileName, String optionName, String defaultValue, String splitValue)
    {
    	Boolean gotLine; // Verification variable
    	String[] returnValue = new String[100]; // Settings max at 100 values
    	String curLine;
    	
    	gotLine = false; // We don't have the settings found yet
    	
    	if( new File(fileName).exists() == false )
    	{
    		return( new String("File not found.ZQX").split("ZQX") );
    	}
		if(splitValue.equals("")) {
			splitValue = "afs4wfa3waawfa3dogrsijkge5ssioeguhwar3awwa3rar";
		}
    	
        try {
        	// Get the line from the file
			FileInputStream fstream = new FileInputStream(fileName);
			BufferedReader in = new BufferedReader(new InputStreamReader(fstream));
			
			while(in.ready()) 
			{
				curLine = in.readLine().toString();
				if(curLine.split("=", 2)[0].equalsIgnoreCase(optionName)) {
					returnValue = new String[100];
					returnValue = curLine.split("=", 2)[1].split(splitValue);
					gotLine = true;
				}
			}
			
			in.close();
			
			// If the line does not exist, create it
			if(!gotLine) 
			{
                returnValue = defaultValue.split(splitValue);
                FileOutputStream out;
                PrintStream p;
                try {
	                out = new FileOutputStream(fileName, true);
	                p = new PrintStream( out );
	                p.println (optionName+"="+defaultValue);
	                p.close();
                } catch (Exception e) {
                	logOutput("Error writing to file");
                }
			}
		}
        catch (Exception e) {
        	logOutput("-=-");
        	logOutput("File input error: "+e.toString());
        	logOutput("File input error: "+e.getStackTrace().toString());
        	logOutput("-=-");
		}
		finally {
		}
		
		return returnValue;
    }

    public void logOutput(String text)
    {
    	System.out.println( "[" + pluginName + "]: " + text );
    }
}
